#include <unistd.h>

#include <errno.h>
#include <stdio.h>

#include <hipc/scfg.h>
#include <hipc/posix/usoc.h>
#include <hipc/usoc.h>
#include <hipc/usocutils.h>

#include "CFGled.h"
#include "CFGled_scfg.h"

static struct STRUprm prm;

void hipcsvsLedUpdate(struct STRUprm *prm_p)
{
    fprintf(stderr, "led:%d\n", prm_p->led);

    return;
}

enum HIPC_errno hipcsvsLedRun(struct HIPC_usoc *const so_p)
{
    hipc_msgd mdR;
    hipc_msgd mdS;
    enum HIPC_errno retval;

    for (;;) {
        retval =
            hipcUsocRead(so_p, hipcMsgdHdrPtr(&mdR), HIPC_MSGHDR_SIZE);
        if (HIPC_SUCCESS != retval) {
            return retval;
        }

        switch (hipcMsgdType(&mdR)) {
        case HIPC_MESSAGE_TYPE_BYE:
            return HIPC_SUCCESS;
        case HIPC_MESSAGE_TYPE_GET:
            retval = hipcGenmdSuccessToGet(&mdS, &mdR, &prm, sizeof(prm));
            if (HIPC_SUCCESS != retval) {
                return retval;
            }
            break;
        case HIPC_MESSAGE_TYPE_PUT:
            retval = hipcMsgdSetBdyForPut(&mdR, &prm, sizeof(prm));
            if (HIPC_SUCCESS != retval) {
                return retval;
            }
            retval = hipcUsocRead(so_p, hipcMsgdBdyPtr(&mdR),
                                  hipcMsgdBdySize(&mdR));
            if (HIPC_SUCCESS != retval) {
                return retval;
            }
            hipcGenmdSuccess(&mdS);
            hipcsvsLedUpdate(&prm);
            break;
        default:
            return HIPC_ERR_S_INV_MSG_TYPE;
        }
        retval = hipcUsocWriteMsgd(so_p, &mdS);
        if (HIPC_SUCCESS != retval) {
            return retval;
        }
    }
}

void hipcMain(struct HIPC_usoc *const so_p)
{
    hipc_msgd mdQ;
    enum HIPC_errno retval;

    retval = hipcUsocAcceptSimple(so_p, &HIPCUD_CFGled_SCFG);
    if (HIPC_SUCCESS == retval) {
        retval = hipcsvsLedRun(so_p);
    }

    hipcGenmdQuit(&mdQ, retval, NULL);

    if ((HIPC_ERR_S_IO_READ != retval) && (HIPC_ERR_S_IO_WRITE != retval)) {
        hipcUsocWriteMsgd(so_p, &mdQ);
    }

    return;
}

int main(void)
{
    struct HIPC_usoc usoc;

    prm.led = 0;
    hipcPosixUsocInit(&usoc, STDIN_FILENO, STDOUT_FILENO);
    hipcMain(&usoc);

    fprintf(stderr, "end\n");

    return 0;
}
