#include <string.h>

#include <hipc/scfg.h>
#include <hipc/usoc.h>
#include <hipc/usocutils.h>

#include "CFGled.h"
#include "CFGled_scfg.h"
#include "hipcsvs_led.h"

static struct STRUprm prm;

void hipcsvsLedInit(void)
{
    prm.led = 0;
    hipcsvsLedUpdate(&prm);

    return;
}

enum HIPC_errno hipcsvsLedRun(struct HIPC_usoc *const so_p,
                              hipc_msgd * const mdq_p)
{
    hipc_msgd mdR;
    hipc_msgd mdS;
    enum HIPC_errno retval;

    for (;;) {
        retval =
            hipcUsocRead(so_p, hipcMsgdHdrPtr(&mdR), HIPC_MSGHDR_SIZE);
        if (HIPC_SUCCESS != retval) {
            return hipcGenmdQuit(mdq_p, retval, "Header");
        }

        switch (hipcMsgdType(&mdR)) {
        case HIPC_MESSAGE_TYPE_BYE:
            return hipcGenmdQuit(mdq_p, HIPC_SUCCESS, NULL);
        case HIPC_MESSAGE_TYPE_GET:
            if (HIPCUD_STRUprm != hipcMsgdStruno(&mdR)) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_STRUNO,
                                     "GETc0");
            }
            retval = hipcGenmdSuccessToGet(&mdS, &mdR, &prm, sizeof(prm));
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "GETr");
            }
            break;
        case HIPC_MESSAGE_TYPE_PUT:
            if (HIPCUD_STRUprm != hipcMsgdStruno(&mdR)) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_STRUNO,
                                     "PUTc0");
            }
            retval = hipcMsgdSetBdyForPut(&mdR, &prm, sizeof(prm));
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "PUTs");
            }
            retval = hipcUsocRead(so_p, hipcMsgdBdyPtr(&mdR),
                                  hipcMsgdBdySize(&mdR));
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "PUTb");
            }
            if (1 < prm.led) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_UDATA,
                                     "1 < prm.led");
            }
            hipcGenmdSuccess(&mdS);
            hipcsvsLedUpdate(&prm);
            break;
        default:
            return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_MSG_TYPE, NULL);
        }
        retval = hipcUsocWriteMsgd(so_p, &mdS);
        if (HIPC_SUCCESS != retval) {
            return hipcGenmdQuit(mdq_p, retval, "RESP");
        }
    }
}

void hipcMain(struct HIPC_usoc *const so_p)
{
    hipc_msgd mdQ;
    enum HIPC_errno retval;

    hipcGenmdQuit(&mdQ, HIPC_ERR_S_V_UNINIT_MSG, NULL);

    retval = hipcUsocAcceptSimple(so_p, &HIPCUD_CFGled_SCFG);
    if (HIPC_SUCCESS != retval) {
        hipcGenmdQuit(&mdQ, retval, "Failed to accept");
    } else {
        retval = hipcsvsLedRun(so_p, &mdQ);
    }

    if ((HIPC_ERR_S_IO_READ != retval) && (HIPC_ERR_S_IO_WRITE != retval)) {
        hipcUsocWriteMsgd(so_p, &mdQ);
    }

    return;
}
