#include <hipc.h>

#include "CFGrt.h"
#include "hipcsvs_rt.h"

#define LOOP_PERIOD 1000
#define LED_PIN 13

int hipcUsocReadReady(struct HIPC_usoc *so_p)
{
    if (0 < Serial.available()) {
        return 1;
    } else {
        return 0;
    }
}

taskLoop(rt_task)
{
    struct HIPC_pbank *const pbank_p = &hipcsvsRt_pbank;
    struct HIPC_queue *const queue_p = &hipcsvsRt_queue;
    struct STRUprm const *prm_p;
    struct STRUlog *log_p;
    uint8_t output;
    unsigned int cnt = 0;
    portTickType xLastWakeTime;

    prm_p = static_cast < const struct STRUprm *>
        (hipcPbankGetReadOnlyPage(pbank_p));
    xLastWakeTime = xTaskGetTickCount();

    for (;;) {
        if (0 == cnt) {
            if (1 == hipcPbankSwitchAccept(pbank_p)) {
                prm_p = static_cast < const struct STRUprm *>
                    (hipcPbankGetReadOnlyPage(pbank_p));
                cnt = prm_p->size;
            }
        }

        if (0 < cnt) {
            output = prm_p->pattern[prm_p->size - cnt];
            digitalWrite(LED_PIN, output);

            log_p =
                static_cast < struct STRUlog *>(hipcQueueFull(queue_p));
            if (NULL != log_p) {
                log_p->tick = xLastWakeTime;
                log_p->output = output;
                hipcQueuePush(queue_p);
            }
            --cnt;
        }
        vTaskDelayUntil(&xLastWakeTime, LOOP_PERIOD);
    }
}

void setup()
{
    pinMode(LED_PIN, OUTPUT);

    initMainLoopPriority(LOW_PRIORITY);
    initMainLoopStackSize(192);

    hipcsvsRtInit();
    createTaskLoop(rt_task, HIGH_PRIORITY);
}


void loop()
{
    Serial.begin(9600);
    hipcMain(NULL);
    Serial.flush();
    Serial.end();
    delay(1000);
}
