#include <string.h>

#include <hipc/scfg.h>
#include <hipc/usoc.h>
#include <hipc/usocutils.h>

#include "CFGrt.h"
#include "CFGrt_scfg.h"
#include "hipcsvs_rt.h"

#define PBANK_PAGE_SIZE sizeof(struct STRUprm)

struct HIPC_pbank hipcsvsRt_pbank;

void hipcsvsRtInit(void)
{
    static struct STRUprm bufprm[2];

    bufprm[0].led = 0;
    hipcPbankInit(&hipcsvsRt_pbank, bufprm, PBANK_PAGE_SIZE);

    return;
}

enum HIPC_errno hipcsvsRtRun(struct HIPC_usoc *const so_p,
                             hipc_msgd * const mdq_p)
{
    hipc_msgd mdR;
    hipc_msgd mdS;
    enum HIPC_errno retval;
    struct HIPC_pbank *const pbank_p = &hipcsvsRt_pbank;
    struct STRUprm *prm_p;

    for (;;) {
        retval =
            hipcUsocRead(so_p, hipcMsgdHdrPtr(&mdR), HIPC_MSGHDR_SIZE);
        if (HIPC_SUCCESS != retval) {
            return hipcGenmdQuit(mdq_p, retval, "Header");
        }

        switch (hipcMsgdType(&mdR)) {
        case HIPC_MESSAGE_TYPE_BYE:
            return hipcGenmdQuit(mdq_p, HIPC_SUCCESS, NULL);
        case HIPC_MESSAGE_TYPE_GET:
            if (HIPCUD_STRUprm != hipcMsgdStruno(&mdR)) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_STRUNO,
                                     "GETc0");
            }
            if (HIPC_PBANK_STAT_GROUND != hipcPbankGetStat(pbank_p)) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_PBANK_STAT_MISMATCH,
                                     "GETc1");
            }
            retval =
                hipcGenmdSuccessToGet(&mdS, &mdR,
                                      hipcPbankGetWritablePage(pbank_p),
                                      PBANK_PAGE_SIZE);
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "GETr");
            }
            break;
        case HIPC_MESSAGE_TYPE_PUT:
            if (HIPCUD_STRUprm != hipcMsgdStruno(&mdR)) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_STRUNO,
                                     "PUTc0");
            }
            if (HIPC_PBANK_STAT_GROUND != hipcPbankGetStat(pbank_p)) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_PBANK_STAT_MISMATCH,
                                     "PUTc1");
            }

            prm_p = hipcPbankGetWritablePage(pbank_p);
            retval = hipcMsgdSetBdyForPut(&mdR, prm_p, PBANK_PAGE_SIZE);
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "PUTs");
            }
            retval = hipcUsocRead(so_p, hipcMsgdBdyPtr(&mdR),
                                  hipcMsgdBdySize(&mdR));
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "PUTb");
            }

            if (1 < prm_p->led) {
                return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_UDATA,
                                     "1 < prm.led");
            }

            retval = hipcPbankSwitchReqWaitCmpl(pbank_p);
            if (HIPC_SUCCESS != retval) {
                return hipcGenmdQuit(mdq_p, retval, "pbank");
            }

            hipcGenmdSuccess(&mdS);
            break;
        default:
            return hipcGenmdQuit(mdq_p, HIPC_ERR_S_INV_MSG_TYPE, NULL);
        }
        retval = hipcUsocWriteMsgd(so_p, &mdS);
        if (HIPC_SUCCESS != retval) {
            return hipcGenmdQuit(mdq_p, retval, "RESP");
        }
    }
}

void hipcMain(struct HIPC_usoc *const so_p)
{
    hipc_msgd mdQ;
    enum HIPC_errno retval;

    hipcGenmdQuit(&mdQ, HIPC_ERR_S_V_UNINIT_MSG, NULL);

    retval = hipcUsocAcceptSimple(so_p, &HIPCUD_CFGrt_SCFG);
    if (HIPC_SUCCESS != retval) {
        hipcGenmdQuit(&mdQ, retval, "Failed to accept");
    } else {
        retval = hipcsvsRtRun(so_p, &mdQ);
    }

    if ((HIPC_ERR_S_IO_READ != retval) && (HIPC_ERR_S_IO_WRITE != retval)) {
        hipcUsocWriteMsgd(so_p, &mdQ);
    }

    return;
}
