#include <stdint.h>

#include <hipc.h>

/**
 * Returns message size determined by a message header.
 * Only GET type is treated specially.
 *
 * @param msghdr HIPC message header
 * @returns message size
 */
unsigned char hipcMsghdrBdysize(unsigned char const *const msghdr)
{
    if (HIPC_MESSAGE_TYPE_GET == msghdr[HIPC_MSGHDR_BYTE_TYPE]) {
        return 0;
    }
    return msghdr[HIPC_MSGHDR_BYTE_BDYSZ];
}

/**
 * Returns endian of the system.
 *
 * @returns ASCII character value according to endian; 'B': Big, 'l': little, 'o': other
 */
unsigned char hipcEndian(void)
{
    uint16_t tmp;
    unsigned char endian;

    tmp = 0x0102U;
    if ((0x01 == ((unsigned char *) &tmp)[0]) &&
        (0x02 == ((unsigned char *) &tmp)[1])) {
        endian = 0x42;          /* 'B' */
    } else if ((0x02 == ((unsigned char *) &tmp)[0]) &&
               (0x01 == ((unsigned char *) &tmp)[1])) {
        endian = 0x6c;          /* 'l' */
    } else {
        endian = 0x6f;          /* 'o' */
    }
    return endian;
}
