#include <hipc/msg.h>

/**
 * Returns total size of a HIPC message, which is the header size plus the body size.
 *
 * @param msg HIPC message
 * @returns size of the HIPC message
 */
size_t hipcMsgTotalSize(const hipc_msg msg)
{
    return hipcMsghdrBdysize(msg) + HIPC_MSGHDR_SIZE;
}

/**
 * Returns body size of a HIPC message.
 *
 * @param msg HIPC message
 * @returns body size of the HIPC message
 */
unsigned char hipcMsgBdySize(const hipc_msg msg)
{
    return hipcMsghdrBdysize(msg);
}

/**
 * Returns a pointer to a body of a HIPC message.
 *
 * @param msg HIPC message
 * @returns pointer to the body of the HIPC message
 */
const void *hipcMsgBdyPtr(const hipc_msg msg)
{
    return &msg[HIPC_MSGHDR_SIZE];
}

/**
 * Returns type of a HIPC message.
 *
 * @param msg HIPC message
 * @returns type of the HIPC message
 */
unsigned char hipcMsgType(const hipc_msg msg)
{
    return msg[HIPC_MSGHDR_BYTE_TYPE];
}

/**
 * Returns offset of a HIPC message.
 *
 * @param msg HIPC message
 * @returns offset of the HIPC message
 */
unsigned char hipcMsgOffset(const hipc_msg msg)
{
    return msg[HIPC_MSGHDR_BYTE_OFFSET];
}

/**
 * Returns structure number of a HIPC message.
 *
 * @param msg HIPC message
 * @returns structure number of the HIPC message
 */
unsigned char hipcMsgStruno(const hipc_msg msg)
{
    return msg[HIPC_MSGHDR_BYTE_STRUNO];
}

/**
 * Returns range size of a HIPC message.
 *
 * @param msg HIPC message
 * @returns range size of the HIPC message
 */
unsigned char hipcMsgRngsize(const hipc_msg msg)
{
    return msg[HIPC_MSGHDR_BYTE_BDYSZ];
}
