#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <mockio.h>

void mockIOInit(struct MockIO *mio_p, const struct MockIOOp *ops,
                const unsigned int nops)
{
    mio_p->ops = ops;
    mio_p->nops = nops;
    mio_p->i = 0;
    mio_p->mock_errno = MOCK_SUCESS;

    return;
}

int mockIOCheckCompletion(struct MockIO *mio_p)
{
    if (MOCK_SUCESS != mio_p->mock_errno) {
        return -1;
    }
    if (mio_p->nops != mio_p->i) {
        return 0;
    }
    return 1;
}

enum HIPC_errno mockIORead(struct MockIO *mio_p, void *buf,
                           const size_t size)
{
    enum HIPC_errno retval;

    if (mio_p->nops <= mio_p->i) {
        fprintf(stderr, "ERROR: mockio overrun at i==%d in read\n",
                mio_p->i);
        mio_p->mock_errno = MOCK_ERR_OVERRUN;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }
    if (0 != mio_p->ops[mio_p->i].code) {
        fprintf(stderr,
                "ERROR: mockio code mismatch: i==%d Eepected/Actual %d/0\n",
                mio_p->i, mio_p->ops[mio_p->i].code);
        mio_p->mock_errno = MOCK_ERR_OPCODE;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }
    if (size != mio_p->ops[mio_p->i].size) {
        fprintf(stderr,
                "ERROR: mockio size mismatch: i==%d Eepected/Actual %lu/%lu\n",
                mio_p->i, mio_p->ops[mio_p->i].size, size);
        mio_p->mock_errno = MOCK_ERR_SIZE;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }

    memcpy(buf, mio_p->ops[mio_p->i].data, size);
    retval = mio_p->ops[mio_p->i].retval;
    ++(mio_p->i);

    return retval;
}

enum HIPC_errno mockIOWrite(struct MockIO *mio_p, const void *buf,
                            const size_t size)
{
    enum HIPC_errno retval;
    unsigned int i;

    if (mio_p->nops <= mio_p->i) {
        fprintf(stderr, "ERROR: mockio overrun at i==%d in write\n",
                mio_p->i);
        mio_p->mock_errno = MOCK_ERR_OVERRUN;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }
    if (1 != mio_p->ops[mio_p->i].code) {
        fprintf(stderr,
                "ERROR: mockio code mismatch: i==%d Eepected/Actual %d/1\n",
                mio_p->i, mio_p->ops[mio_p->i].code);
        mio_p->mock_errno = MOCK_ERR_OPCODE;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }
    if (size != mio_p->ops[mio_p->i].size) {
        fprintf(stderr,
                "ERROR: mockio size mismatch: i==%d Eepected/Actual %lu/%lu\n",
                mio_p->i, mio_p->ops[mio_p->i].size, size);
        mio_p->mock_errno = MOCK_ERR_SIZE;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }

    if (0 != memcmp(buf, mio_p->ops[mio_p->i].data, size)) {
        fprintf(stderr, "ERROR: write contents mismatch: i==%d\n",
                mio_p->i);
        fprintf(stderr, "Expected:\n");
        for (i = 0; i < size; ++i) {
            fprintf(stderr, "0x%02x,",
                    ((unsigned char *) mio_p->ops[mio_p->i].data)[i]);
        }
        fprintf(stderr, "\n");
        fprintf(stderr, "Actual:\n");
        for (i = 0; i < size; ++i) {
            fprintf(stderr, "0x%02x,", ((unsigned char *) buf)[i]);
        }
        fprintf(stderr, "\n");
        mio_p->mock_errno = MOCK_ERR_DATA;
        return HIPC_ERR_TEST_LIB_INTERNAL;
    }

    retval = mio_p->ops[mio_p->i].retval;
    ++(mio_p->i);

    return retval;
}
