#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <hipc/msgd.h>

static void test_hipcMsgdHdrPtr(void)
{
    hipc_msgd md;

    assert(md.header == hipcMsgdHdrPtr(&md));

    return;
}

static void test_hipcMsgdBdySize_hipcMsgdBdyPtr(void)
{
    hipc_msgd md;
    unsigned char body[64];

    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = sizeof(body);
    md.body_p = body;

    md.header[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_PUT;
    assert(sizeof(body) == hipcMsgdBdySize(&md));
    assert(body == hipcMsgdBdyPtr(&md));

    md.header[HIPC_MSGHDR_BYTE_TYPE] = HIPC_MESSAGE_TYPE_GET;
    md.body_p = NULL;
    assert(0 == hipcMsgdBdySize(&md));
    assert(NULL == hipcMsgdBdyPtr(&md));

    return;
}

static void test_hipcMsgdType(void)
{
    hipc_msgd md;

    md.header[HIPC_MSGHDR_BYTE_TYPE] = 11;
    assert(11 == hipcMsgdType(&md));

    return;
}

static void test_hipcMsgdStruno(void)
{
    hipc_msgd md;

    md.header[HIPC_MSGHDR_BYTE_STRUNO] = 127;
    assert(127 == hipcMsgdStruno(&md));

    return;
}

static void test_hipcMsgdOffset(void)
{
    hipc_msgd md;

    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 7;
    assert(7 == hipcMsgdOffset(&md));

    return;
}

static void test_hipcMsgdRngsize(void)
{
    hipc_msgd md;

    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = 5;
    assert(5 == hipcMsgdRngsize(&md));

    return;
}

struct local_mock {
    long m1;
    int m2;
};

static void test_hipcMsgdSetBdyForPut(void)
{
    hipc_msgd md;
    struct local_mock lm;

    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = 0;

    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 0;
    assert(HIPC_SUCCESS == hipcMsgdSetBdyForPut(&md, &lm, sizeof(lm)));
    assert(&lm == md.body_p);

    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 1;
    assert(HIPC_SUCCESS == hipcMsgdSetBdyForPut(&md, &lm, sizeof(lm)));
    assert((unsigned char *) (&lm) + 1 == md.body_p);

    md.header[HIPC_MSGHDR_BYTE_OFFSET] = offsetof(struct local_mock, m2);
    assert(HIPC_SUCCESS == hipcMsgdSetBdyForPut(&md, &lm, sizeof(lm)));
    assert(&(lm.m2) == md.body_p);


    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = sizeof(lm);
    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 0;
    assert(HIPC_SUCCESS == hipcMsgdSetBdyForPut(&md, &lm, sizeof(lm)));
    assert(&lm == md.body_p);

    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = sizeof(lm);
    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 1;
    assert(HIPC_ERR_S_INV_RW_RANGE ==
           hipcMsgdSetBdyForPut(&md, &lm, sizeof(lm)));

    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = 255;
    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 255;
    assert(HIPC_SUCCESS == hipcMsgdSetBdyForPut(&md, &lm, 510));

    md.header[HIPC_MSGHDR_BYTE_BDYSZ] = 255;
    md.header[HIPC_MSGHDR_BYTE_OFFSET] = 255;
    assert(HIPC_ERR_S_INV_RW_RANGE == hipcMsgdSetBdyForPut(&md, &lm, 509));

    return;
}

static void test_hipcGenmdSuccessToGet(void)
{
    struct local_mock lm;
    hipc_msgd mdS;

    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET, 0, 0, 0}, NULL };
        assert(HIPC_SUCCESS ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, sizeof(lm)));
        assert(HIPC_MESSAGE_TYPE_SUCCESS ==
               mdS.header[HIPC_MSGHDR_BYTE_TYPE]);
        assert(0 == mdS.header[HIPC_MSGHDR_BYTE_STRUNO]);
        assert(0 == mdS.header[HIPC_MSGHDR_BYTE_OFFSET]);
        assert(0 == mdS.header[HIPC_MSGHDR_BYTE_BDYSZ]);
        assert(&lm == mdS.body_p);
    }
    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET, 1, 2, 3}, NULL };
        assert(HIPC_SUCCESS ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, sizeof(lm)));
        assert(HIPC_MESSAGE_TYPE_SUCCESS ==
               mdS.header[HIPC_MSGHDR_BYTE_TYPE]);
        assert(1 == mdS.header[HIPC_MSGHDR_BYTE_STRUNO]);
        assert(2 == mdS.header[HIPC_MSGHDR_BYTE_OFFSET]);
        assert(3 == mdS.header[HIPC_MSGHDR_BYTE_BDYSZ]);
        assert((unsigned char *) (&lm) + 2 == mdS.body_p);
    }
    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET,
                           5,
                           offsetof(struct local_mock, m2),
                           sizeof(lm.m2)}, NULL };
        assert(HIPC_SUCCESS ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, sizeof(lm)));
        assert(HIPC_MESSAGE_TYPE_SUCCESS ==
               mdS.header[HIPC_MSGHDR_BYTE_TYPE]);
        assert(5 == mdS.header[HIPC_MSGHDR_BYTE_STRUNO]);
        assert(offsetof(struct local_mock, m2) ==
               mdS.header[HIPC_MSGHDR_BYTE_OFFSET]);
        assert(sizeof(lm.m2) == mdS.header[HIPC_MSGHDR_BYTE_BDYSZ]);
        assert(&(lm.m2) == mdS.body_p);
    }
    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET, 0, 0, sizeof(lm)}, NULL };
        assert(HIPC_SUCCESS ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, sizeof(lm)));
        assert(HIPC_MESSAGE_TYPE_SUCCESS ==
               mdS.header[HIPC_MSGHDR_BYTE_TYPE]);
        assert(0 == mdS.header[HIPC_MSGHDR_BYTE_STRUNO]);
        assert(0 == mdS.header[HIPC_MSGHDR_BYTE_OFFSET]);
        assert(sizeof(lm) == mdS.header[HIPC_MSGHDR_BYTE_BDYSZ]);
        assert(&lm == mdS.body_p);
    }
    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET, 0, 1, sizeof(lm)}, NULL };
        assert(HIPC_ERR_S_INV_RW_RANGE ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, sizeof(lm)));
    }
    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET, 0, 255, 255}, NULL };
        assert(HIPC_SUCCESS ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, 510));
    }
    {
        hipc_msgd mdR = { {HIPC_MESSAGE_TYPE_GET, 0, 255, 255}, NULL };
        assert(HIPC_ERR_S_INV_RW_RANGE ==
               hipcGenmdSuccessToGet(&mdS, &mdR, &lm, 509));
    }

    return;
}

static void test_hipcGenmdUnstru(void)
{
    hipc_msgd md;
    char str[256];
    char abc[] = "abc";

    hipcGenmdUnstru(&md, HIPC_MESSAGE_TYPE_QUIT, HIPC_SUCCESS, NULL, 0);
    assert(HIPC_MESSAGE_TYPE_QUIT == md.header[HIPC_MSGHDR_BYTE_TYPE]);
    assert(HIPC_UNSTRU == md.header[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(HIPC_SUCCESS == md.header[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(0 == md.header[HIPC_MSGHDR_BYTE_BDYSZ]);
    assert(NULL == md.body_p);

    hipcGenmdUnstru(&md, HIPC_MESSAGE_TYPE_QUIT, HIPC_ERR_S_INV_UDATA, str, 256);
    assert(HIPC_MESSAGE_TYPE_QUIT == md.header[HIPC_MSGHDR_BYTE_TYPE]);
    assert(HIPC_UNSTRU == md.header[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(HIPC_ERR_S_INV_UDATA == md.header[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(255 == md.header[HIPC_MSGHDR_BYTE_BDYSZ]);
    assert(str == md.body_p);

    hipcGenmdQuit(&md, HIPC_ERR_S_INV_UDATA, abc);
    assert(HIPC_MESSAGE_TYPE_QUIT == md.header[HIPC_MSGHDR_BYTE_TYPE]);
    assert(HIPC_UNSTRU == md.header[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(HIPC_ERR_S_INV_UDATA == md.header[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(4 == md.header[HIPC_MSGHDR_BYTE_BDYSZ]);
    assert(abc == md.body_p);

    hipcGenmdSuccess(&md);
    assert(HIPC_MESSAGE_TYPE_SUCCESS == md.header[HIPC_MSGHDR_BYTE_TYPE]);
    assert(HIPC_UNSTRU == md.header[HIPC_MSGHDR_BYTE_STRUNO]);
    assert(HIPC_SUCCESS == md.header[HIPC_MSGHDR_BYTE_OFFSET]);
    assert(0 == md.header[HIPC_MSGHDR_BYTE_BDYSZ]);
    assert(NULL == md.body_p);

    return;
}

int main(int argc, char *argv[])
{
    if (2 != argc) {
        fprintf(stderr, "Usage: %s test_id\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    switch (argv[1][0]) {
    case 'a':
        test_hipcMsgdHdrPtr();
        break;
    case 'b':
        test_hipcMsgdBdySize_hipcMsgdBdyPtr();
        break;
    case 'c':
        test_hipcMsgdType();
        break;
    case 'd':
        test_hipcMsgdStruno();
        break;
    case 'e':
        test_hipcMsgdOffset();
        break;
    case 'f':
        test_hipcMsgdRngsize();
        break;
    case 'g':
        test_hipcMsgdSetBdyForPut();
        break;
    case 'h':
        test_hipcGenmdSuccessToGet();
        break;
    case 'i':
        test_hipcGenmdUnstru();
        break;
    default:
        fprintf(stderr, "Invalid test_id: %s \n", argv[1]);
        exit(EXIT_FAILURE);
    }

    return 0;
}
