#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <hipc/pbank.h>

struct page {
    int a;
    char b;
};

static void test_hipcPbank(void)
{
    struct HIPC_pbank bank;
    struct page buf[2];
    struct page *pageRW;
    const struct page *pageRO;

    buf[0].a = 2;
    buf[0].b = 3;
    hipcPbankInit(&bank, buf, sizeof(buf[0]));
    assert(HIPC_PBANK_STAT_GROUND == hipcPbankGetStat(&bank));
    assert(sizeof(buf[0]) == hipcPbankGetPageSize(&bank));

    pageRW = hipcPbankGetWritablePage(&bank);
    assert(NULL != pageRW);
    assert(2 == pageRW->a);
    assert(3 == pageRW->b);

    pageRW->a = 5;

    assert(0 == hipcPbankSwitchAccept(&bank));
    pageRO = hipcPbankGetReadOnlyPage(&bank);
    assert(NULL != pageRO);
    assert(2 == pageRO->a);
    assert(3 == pageRO->b);

    assert(HIPC_SUCCESS == hipcPbankSwitchRequest(&bank));
    assert(HIPC_PBANK_STAT_REQUESTED == hipcPbankGetStat(&bank));

    assert(1 == hipcPbankSwitchAccept(&bank));
    pageRO = hipcPbankGetReadOnlyPage(&bank);
    assert(NULL != pageRO);
    assert(5 == pageRO->a);
    assert(3 == pageRO->b);
    assert(HIPC_PBANK_STAT_ACCEPTED == hipcPbankGetStat(&bank));

    pageRW = hipcPbankGetWritablePage(&bank);
    assert(NULL != pageRW);
    assert(2 == pageRW->a);
    assert(3 == pageRW->b);

    assert(HIPC_SUCCESS == hipcPbankSwitchComplete(&bank));
    assert(HIPC_PBANK_STAT_GROUND == hipcPbankGetStat(&bank));
    assert(5 == pageRW->a);
    assert(3 == pageRW->b);

    return;
}

static void test_hipcPbank_err(void)
{
    struct HIPC_pbank bank;
    struct page buf[2];

    hipcPbankInit(&bank, buf, sizeof(buf[0]));
    assert(HIPC_PBANK_STAT_GROUND == hipcPbankGetStat(&bank));

    assert(HIPC_ERR_S_PBANK_STAT_MISMATCH ==
           hipcPbankSwitchComplete(&bank));

    assert(HIPC_SUCCESS == hipcPbankSwitchRequest(&bank));
    assert(HIPC_PBANK_STAT_REQUESTED == hipcPbankGetStat(&bank));

    assert(HIPC_ERR_S_PBANK_STAT_MISMATCH ==
           hipcPbankSwitchRequest(&bank));
    assert(HIPC_ERR_S_PBANK_STAT_MISMATCH ==
           hipcPbankSwitchComplete(&bank));

    assert(1 == hipcPbankSwitchAccept(&bank));
    assert(HIPC_PBANK_STAT_ACCEPTED == hipcPbankGetStat(&bank));

    assert(HIPC_ERR_S_PBANK_STAT_MISMATCH ==
           hipcPbankSwitchRequest(&bank));

    assert(HIPC_SUCCESS == hipcPbankSwitchComplete(&bank));
    assert(HIPC_PBANK_STAT_GROUND == hipcPbankGetStat(&bank));

    return;
}

int main(int argc, char *argv[])
{
    if (2 != argc) {
        fprintf(stderr, "Usage: %s test_id\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    switch (argv[1][0]) {
    case 'a':
        test_hipcPbank();
        break;
    case 'b':
        test_hipcPbank_err();
        break;
    default:
        fprintf(stderr, "Invalid test_id: %s \n", argv[1]);
        exit(EXIT_FAILURE);
    }

    return 0;
}
